function varargout=buildpartmesh(inputfile,np,varargin)
% FUNCTION fc_oogmsh.gmsh.buildpartmesh
%   Builds gmsh partitioned mesh file from a mesh file or a .geo file.
%
% USAGE:
%   pmfile=fc_oogmsh.gmsh.buildpartmesh(inputfile,np)
%   pmfile=fc_oogmsh.gmsh.buildpartmesh(inputfile,np,key,value)
%
% INPUT:
%   inputfile: name of the input file, mesh file or .geo file. 
%   np       : number of partitions.
%
%   optional parameters as key/value pairs:
%     'geodir'  : to specify the directory of the geofile.
%     'meshdir' : to specify the directory of the input mesh file.
%     'savefile': to specify the name of the output partitioned mesh file.
%     'savedir' : to specify the directory of the output partitioned mesh file.
%     'd': to specify space dimension 2(default) or 3.
%     'N': parameter used in the .geo file for mesh refinement.
%     'Loptions' : gmsh list of options as a cell array. must contains '-saveall'
%   and all optional parameters of fc_oogmsh.gmsh.run
%
% OUTPUT:
%   pmfile: name of partitioned mesh file as string.
%
% SAMPLES:
%
% Copyright (C) 2017-2023 F. Cuvelier
  assert((round(np)==np) & (np>1))
  %if ~isfileexists(meshfile),error('file %s not found!',meshfile);end
  
  env=fc_oogmsh.environment;
  format=fc_oogmsh.gmsh.setformat();
  p = inputParser;
  p.KeepUnmatched=true; 
  p.addParameter('geodir',env.geo_dir,@ischar); % select labels to draw
  p.addParameter('meshdir',env.mesh_dir,@ischar);
  p.addParameter('savefile','',@ischar);
  p.addParameter('savedir','',@ischar);
  p.addParameter('check',false,@islogical);
  p.addParameter('Loptions',{'-saveall'},@iscell);
  p.addParameter('MshFileVersion',format,@(x) ismember(x,fc_oogmsh.gmsh.getMSHfile_formats()));
  p.addParameter('N',10,@isscalar);
  p.addParameter('d',2,@(x) ismember(x,[2,3]));
  p.parse(varargin{:});
  R=p.Results;
  varargin=fc_tools.utils.deleteCellOptions(varargin,p.Parameters);
  inputfile=fc_oogmsh.gmsh.findMeshOrGeofile(inputfile,R.geodir,R.meshdir);
  [pathstr,name,ext] = fileparts(inputfile);
  if strcmp(R.MshFileVersion,'2.2')
    ver=fc_oogmsh.gmsh.version;
    if ~ismember(ver,{'2.16.0','3.0.6'})
      fprintf('[fc-oogmsh] Building partitioned MSH 2.2 files only allowed with GMSH 2.16.0 and 3.0.6 :\n');
      fprintf('            -> <fc-oogmsh> uses negative tags for interfaces...\n');
      error('Only GMSH version 2.16.0 or 3.0.6 can be used with 2.2 MSH file format')
    end
  end
  if sum(cell2mat(cellfun(@(x) numel(strfind('-saveall',x)),R.Loptions,'UniformOutput',false)))==0
    R.Loptions={'-saveall',R.Loptions{:}}; % 
  end
  
  if strcmp(ext,'.geo')
    if isempty(R.savedir), R.savedir=R.meshdir;end
    pmfile=[R.savedir,filesep,name,sprintf('-%d-part%d.msh',R.N,np)];
    pmfile=fc_oogmsh.gmsh.run(inputfile,'d',R.d,'options',sprintf('-part %d -setnumber N %d ',np,R.N), ...
                    'meshfile',pmfile,'MshFileVersion',R.MshFileVersion,'Loptions',R.Loptions,varargin{:});
    %pmfile=fc_oogmsh.gmsh.run(inputfile,'d',R.d,'options',sprintf('-saveall -part %d -setnumber N %d ',np,R.N), ...
    %                'meshfile',pmfile,varargin{:});
  else
    if isempty(R.savedir) 
      if isempty(pathstr), R.savedir=R.meshdir;else R.savedir=pathstr; end
    end
    pmfile=[R.savedir,filesep,name,sprintf('-part%d.msh',np)];
    pmfile=fc_oogmsh.gmsh.run(inputfile,'d',R.d,'options',sprintf('-part %d ',np), ...
                    'meshfile',pmfile,'MshFileVersion',R.MshFileVersion,'Loptions',R.Loptions,varargin{:});
%      pmfile=[R.savedir,filesep,name,sprintf('-part%d.msh',np)];
%      pmfile=fc_oogmsh.gmsh.run(inputfile,'d',R.d,'options',sprintf('-saveall -part %d ',np), ...
%                      'meshfile',pmfile,varargin{:});
  end
  
  if R.check
    [status,result]=fc_oogmsh.gmsh.checkmesh(pmfile);
    if status==0
      fprintf('[fc-oogmsh] Check mesh : OK\n')
    else
      fprintf('[fc-oogmsh] Check mesh : Failed\n')
    end
  end
  if nargout>=1, varargout{1}=pmfile;end
  if nargout==2, varargout{2}=inputfile;end
end


