function varargout=plot(Th,u,varargin)
% FUNCTION plot 
%   Plot scalar data values on the siMesh object.
%
% USAGE
%   plot(Th,u) where Th is a siMesh object and u is
%   a vector of length Th.nq which contains the scalar data values 
%   on each vertices of the siMesh object.
%
%   fc_siplt.plot(Th,u,key,value,...) allows additional key/value pairs to be 
%     used when displaying u. The key strings could be
%
%     'd'      to specify the d-simplicial meshes. Default d=Th.d
%     'labels' to specify the labels of the elementary meshes. 
%              Default [] i.e all.
%     'plane'   (only in dimension 2), if true display is done in x0y plane,
%        otherwise the z-axe is the scalar data values. Default false.
%
%   or any of the plot options depending of Th.d and Th.dim. Same key/pairs as
%
%     - patch   function for d=3 and dim=3 (no face)
%     - patch   function for d=3 and dim=2
%     - patch   function for d=3 and dim=1
%     - patch   function for d=2 and dim=2 with 'plane'==true
%     - trisurf function for d=2 and dim=2 with 'plane'==false
%     - patch   function for d=2 and dim=1 
%     - plot    function for d=1 and dim=1 
%
% SAMPLES
%   Th=fc_simesh.hypercube(3,10);
%   U=Th.eval(@(x,y,z) cos(2*x-y+z).*sin(x+y-3*z));
%   figure(1)
%   fc_siplt.plot(Th,U)
%   figure(2)
%   fc_siplt.plot(Th,U,'d',2,'labels',[1,2,5],'edgecolor','w')
%
%    Copyright (C) 2017-2023 F. Cuvelier
  assert(fc_simesh.is_siMesh(Th))
  %assert(Th.d==3 && Th.dim==3);
  U=Th.eval(u); % also verifies dimensions...
  p = inputParser; 
  p.KeepUnmatched=true; 
  p.PartialMatching=false;
  p.addParamValue('d',Th.d,@isscalar); 
  p.addParamValue('labels',[]); % select labels to draw
  p.addParamValue('merge',true,@islogical);
  p.parse(varargin{:});
  R=p.Results;
  varargin=fc_tools.utils.deleteCellOptions(varargin,p.Parameters);
  varargin=fc_tools.graphics.CheckAlpha(varargin,'fc-simesh',mfilename());
  if isempty(R.labels), idxlab=Th.find(R.d);else idxlab=Th.find(R.d,R.labels);end
  if isempty(idxlab), return;end
  hfig=gcf();
  PrevHold=fc_tools.graphics.SetHoldOn(true);
  if ~PrevHold, clf;hold on;end
  if R.merge
    [q,me,toGlobal]=Th.get_mesh('d',R.d,'labels',R.labels);
    H=fc_graphics4mesh.plot(q,me,U(toGlobal),varargin{:});
  else
    i=1;
    for ilab=[idxlab]
      sTh=Th.sTh{ilab};
      H{i}=fc_graphics4mesh.plot(sTh.q,sTh.me,sTh.getLocalData(u),varargin{:});
      i=i+1;
    end
  end
  fc_tools.graphics.RestoreHold(PrevHold)
  if nargout==1,varargout{1}=H;end
  %figure(hfig); %-> si figure a 'visible','off' alors la rend visible
end
