function varargout=plotmesh(Th,varargin)
% FUNCTION plotmesh 
%   Plots the siMesh object.
%
% USAGE
%   fc_siplt.plotmesh(Th,) or plotmesh(Th) where Th is a siMesh object.
%
%   fc_siplt.plotmesh(Th,key,value,...) allows additional key/value pairs to be 
%     used when displaying the siMesh object. The key strings could be
%
%     'd'      to specify the d-simplicial meshes. Default d=Th.d
%     'labels' to specify the labels of the elementary meshes. 
%              Default [] i.e all.
%     'inlegend' if true display the labels of the elementary meshes in
%              the legend box. Default false.
%     'color'  to specify the color of all the displayed elementary meshes.
%
%   or any of the plotmesh options depending of Th.dim and d. Same key/pairs as
%
%     - patch   function for dim=3 and d=3 (no face)
%     - trimesh function for dim=3 and d=2
%     - plot3   function for dim=3 and d=1
%     - plot3   function for dim=3 and d=0
%     - triplot function for dim=2 and d=2 
%     - plot    function for dim=2 and d=1
%     - plot    function for dim=2 and d=0 
%     - line    function for dim=1 and d=1
%     - plot    function for dim=1 and d=0
%
% SAMPLES
%   Th=fc_simesh.hypercube(3,10);
%   figure(1)
%   fc_siplt.plotmesh(Th,)
%   figure(2)
%   fc_siplt.plotmesh(Th,'d',2,'labels',[1,2,5],'edgecolor','w','legend',true)
%
%    Copyright (C) 2017-2023 F. Cuvelier
  assert(fc_simesh.is_siMesh(Th))
  p = inputParser;
  p.KeepUnmatched=true; 
  %p.PartialMatching=false;
  p.addParamValue('d',Th.d,@isscalar);
  p.addParamValue('labels',[]); % select labels to draw
  p.addParamValue('DisplayName',''); % function for string legend label name or cell of strings (length of labels)
  %p.addParamValue('legend',false,@islogical);
  p.addParamValue('inlegend',false,@islogical);
  p.addParamValue('mouse',true,@islogical);
  p.addParamValue('bounds',false,@islogical);
  p.addParamValue('merge',false,@islogical);
  p.addParamValue('color',[]); % default colors
  p.parse(varargin{:});
  R=p.Results;
  varargin=fc_tools.utils.deleteCellOptions(varargin,p.Parameters);
  varargin=fc_tools.graphics.CheckAlpha(varargin,'fc-siplt',mfilename());
  if isempty(R.labels), idxlab=Th.find(R.d);else idxlab=Th.find(R.d,R.labels);end
  if isempty(idxlab)
    if nargout>=1, varargout{1}=[];end
    if nargout==2, varargout{2}={};end
    return;
  end
  
  if (R.inlegend) || (~isempty(R.DisplayName) )
    if isempty(R.DisplayName) 
      flabName=funLabName(Th,R.d); 
      labName=@(sTh,i) flabName(sTh.label);
    else 
      if iscell(R.DisplayName)
        labName=@(sTh,i) R.DisplayName{i};
      else
        labName=R.DisplayName; %a function handle or string
      end
    end
  else
    if R.merge, labName='';else, labName=@(sTh,i) ''; end
  end
  ColorOption={};
  if ~isempty(R.color), ColorOption={'color',fc_tools.graphics.xcolor.val2rgb(R.color)};end%R.legend=false;end
  hfig=gcf();
  PrevHold=fc_tools.graphics.SetHoldOn(true);
  if R.merge
    [q,me,toGlobal]=Th.get_mesh('d',R.d,'labels',R.labels);
    if R.inlegend
      h=fc_graphics4mesh.plotmesh(q,me,ColorOption{:},'DisplayName',labName,varargin{:});
    else
      h=fc_graphics4mesh.plotmesh(q,me,ColorOption{:},'DisplayName','remove',varargin{:});
    end
  else
    %keyboard
    h=zeros(length(idxlab),1);j=1;
    clegend=cell(length(idxlab),1);
    for ilab=[idxlab]
      if R.bounds 
        if (Th.sTh{ilab}.d==2)
          Thloc=Th.sTh{ilab}.getBoundMesh();
          h(j)=fc_graphics4mesh.plotmesh(Thloc,ColorOption{:},'DisplayName','remove',varargin{:});
          clegend{j}=labName(Th.sTh{ilab}.label);
          j=j+1;
        end
      else
        sTh=Th.sTh{ilab};
        if isempty(R.color), ColorOption={'color',sTh.color};end
        if R.inlegend
          h(j)=fc_graphics4mesh.plotmesh(sTh.q,sTh.me,ColorOption{:},'DisplayName',labName(sTh,ilab),varargin{:});%,'inlegend',R.inlegend);
        else
          h(j)=fc_graphics4mesh.plotmesh(sTh.q,sTh.me,ColorOption{:},'DisplayName','remove',varargin{:});%,'inlegend',R.inlegend);
        end
        j=j+1;
      end
    end
  end
  if Th.dim==3,  view(3);end
  if (R.mouse) && (~fc_tools.comp.isOctave())
    datacursormode on
    dcm = datacursormode(gcf);
    set(dcm,'UpdateFcn',@fc_siplt.showinfo)
  end
  fc_tools.graphics.RestoreHold(PrevHold)
  if nargout>=1, varargout{1}=h;end
  %if nargout==2, varargout{2}=clegend;end
  %figure(hfig); %-> si figure a 'visible','off' alors la rend visible
end
