function varargout=plotmesh_physicals(Th,varargin)
% FUNCTION plotmesh_physicals
%   Plots the siMesh object.
%
% USAGE
%   fc_siplt.plotmesh(Th,) or plotmesh(Th) where Th is a siMesh object.
%
%   fc_siplt.plotmesh(Th,key,value,...) allows additional key/value pairs to be 
%     used when displaying the siMesh object. The key strings could be
%
%     'd'      to specify the d-simplicial meshes. Default d=Th.d
%     'labels' to specify the labels of the elementary meshes. 
%              Default [] i.e all.
%     'inlegend' if true display the labels of the elementary meshes in
%              the legend box. Default false.
%     'color'  to specify the color of all the displayed elementary meshes.
%
%   or any of the plotmesh options depending of Th.dim and d. Same key/pairs as
%
%     - patch   function for dim=3 and d=3 (no face)
%     - trimesh function for dim=3 and d=2
%     - plot3   function for dim=3 and d=1
%     - plot3   function for dim=3 and d=0
%     - triplot function for dim=2 and d=2 
%     - plot    function for dim=2 and d=1
%     - plot    function for dim=2 and d=0 
%     - line    function for dim=1 and d=1
%     - plot    function for dim=1 and d=0
%
% SAMPLES
%   Th=fc_simesh.hypercube(3,10);
%   figure(1)
%   fc_siplt.plotmesh(Th,)
%   figure(2)
%   fc_siplt.plotmesh(Th,'d',2,'labels',[1,2,5],'edgecolor','w','legend',true)
%
%    Copyright (C) 2017-2023 F. Cuvelier
  assert(fc_simesh.is_siMesh(Th))
  p = inputParser;
  p.KeepUnmatched=true; 
  %p.PartialMatching=false;
  p.addParamValue('d',Th.d,@isscalar);
  p.addParamValue('labels',[]); % select physical labels to draw
  p.addParamValue('DisplayName',''); % function for string legend label name or cell of strings (length of labels)
  %p.addParamValue('legend',false,@islogical);
  p.addParamValue('inlegend',false,@islogical);
  p.addParamValue('mouse',true,@islogical);
  %p.addParamValue('bounds',false,@islogical);
  %p.addParamValue('merge',false,@islogical);
  p.addParamValue('color',[]); % default colors
  p.parse(varargin{:});
  R=p.Results;
  varargin=fc_tools.utils.deleteCellOptions(varargin,p.Parameters);
  varargin=fc_tools.graphics.CheckAlpha(varargin,'fc-siplt',mfilename());
  [idxlab,plabels]=Th.find_physicals(R.d,R.labels);
  if isempty(idxlab)
    if nargout>=1, varargout{1}=[];end
    if nargout==2, varargout{2}={};end
    return;
  end
  
  if (R.inlegend) || (~isempty(R.DisplayName) )
    if isempty(R.DisplayName) 
      flabName=Th.funLabName(R.d);
      labName=@(i) [flabName(i),'^{\rm ph}'];
    else 
      if iscell(R.DisplayName)
        labName=@(i) R.DisplayName{i};
      else
        labName=R.DisplayName; %a function handle or string
      end
    end
  else
    %if R.merge, labName='';else, labName=@(sTh,i) ''; end
  end
  ColorOption={};
  if ~isempty(R.color), ColorOption={'color',fc_tools.graphics.xcolor.val2rgb(R.color)};end
  hfig=gcf();
  PrevHold=fc_tools.graphics.SetHoldOn(true);

  labu=unique(plabels);
  j=1;
  for l=labu
    lidx=Th.find_physicals(R.d,l);
    [q,me,toGlobal]=Th.get_mesh('d',R.d,'labels',Th.sThlab(lidx));
    if isempty(R.color), ColorOption={'color',Th.sTh{lidx(1)}.pcolor};end
    if R.inlegend
      h(j)=fc_graphics4mesh.plotmesh(q,me,ColorOption{:},'DisplayName',labName(l),varargin{:});%,'inlegend',R.inlegend);
    else
      h(j)=fc_graphics4mesh.plotmesh(q,me,ColorOption{:},'DisplayName','remove',varargin{:});%,'inlegend',R.inlegend);
    end
    j=j+1;
  end

  if Th.dim==3,  view(3);end
  if (R.mouse) && (~fc_tools.comp.isOctave())
    datacursormode on
    dcm = datacursormode(gcf);
    set(dcm,'UpdateFcn',@fc_siplt.showinfo)
  end
  fc_tools.graphics.RestoreHold(PrevHold)
  if nargout>=1, varargout{1}=h;end
end
