function plot_RGBcolors(rgb,varargin)
%  FUNCTION fc_tools.graphics.plot_RGBcolors
%    displays colors of a n-by-3 RGB colors array with their names
%    if available.
%  USAGE
%    fc_tools.graphics.plot_RGBcolors(rgb)
%      displays colors given by rgb a n-by-3 RGB colors array. The i-th color
%      given by rgb(i,:) is identified by the number i. 
%    fc_tools.graphics.plot_RGBcolors(rgb,'name',name)
%      displays colors given by rgb a n-by-3 RGB colors array. The i-th color
%      given by rgb(i,:) is identified by the number i and its name given by 
%      the string name{i}.
% SAMPLES
%   colors=fc_tools.graphics.selectColors(10);
%   fc_tools.graphics.plot_RGBcolors(colors)
%
%   [name,rgb]=fc_tools.graphics.xcolors.get_from_theme('svg');
%   fc_tools.graphics.plot_RGBcolors(rgb,'name',name)
% Copyright (C) 2017-2023 F. Cuvelier
  p = inputParser;
  p.PartialMatching=false;
  p.addParamValue('position',[100,50,800,800]);
  p.addParamValue('fontsize',10);
  p.addParamValue('name',{},@iscell);
  p.addParamValue('nrmax',30);  % maximum number of rows by figure
  p.addParamValue('nc',3);      % maximum number of columns by figure
  p.addParamValue('tr',20);     % size of truncate name, <=0 full size
  p.parse(varargin{:});
  R=p.Results;
  assert(size(rgb,2)==3)
  N=size(rgb,1);              % nombre total de couleurs
  name=R.name;
  assert( ismember(length(name),[0,N]) )
  if length(name)==0
    name=cell(1,N);
    for i=1:N, name{i}=num2str(i);end
    R.tr=3;
  end
  %isname=(length(name)==N);

  fontsize=R.fontsize;
  nlmax=R.nrmax;                    % nombre de lignes maximum par figure
  n=R.nc;                         % nombres de colonnes 
  mt=fix(N/n)+(rem(N,n)~=0);   % nombre de lignes au total
  m=min(nlmax,mt);             % nombre de lignes par figure
  nf=fix(mt/m)+(rem(mt,m)~=0);  % nombre de figures
  nm=n*m;                      % nombre de couleurs par figure
  
  if isempty(R.position)
    setfig=@(fig) set(fig,'visible','off');
  else
    setfig=@(fig) set(fig,'visible','off','position',R.position);
  end
  tr=R.tr; % nom tronque
  
  fig=figure();
  setfig(fig);
    [~,idx]=max(cellfun(@length, name));
    if tr<=0,tr=length(name{idx(1)});end
    h=text(0,0,name{idx(1)}(1:min(end,tr)),'HorizontalAlignment','left','BackgroundColor','w','Margin',1,'FontSize',fontsize,'Visible','off');
    extent=get(h,'extent');
    clear h;
    ah=extent(4); % hauteur case
    aw=2*ah;      % largeur case
    tw=extent(3); % largeur text
  
  beta=0.3;alpha=0.3;
  sw=alpha*aw;
  sh=beta*ah; % hauteur interligne
  
  k=1;
  for i=1:nf
    if i>1, fig=figure();setfig(fig);end
    OnFigure(fig,rgb,name,N,k,ah,aw,tw,sw,sh,m,n,fontsize,tr)
    k=k+nm;
  end
end

function OnFigure(fig,rgb,name,N,k,ah,aw,tw,sw,sh,m,n,fontsize,tr)
  if isempty(name)
    funname=@(k) num2str(k);
    strname='idx';
  else
    funname=@(k) name{k};
    strname='name';
  end
  ColorRec=@(k,x,y) fill([x,x+aw,x+aw,x],[y,y,y+ah,y+ah],rgb(k,:),'tag',funname(k),'ButtonDownFcn', {@ColorSelected, {strname,k}});
  if fc_tools.comp.isOctave() && (fc_tools.utils.getNumVersion()<=fc_tools.utils.strversion2num('6.4.0')) % to try
     ColorRec=@(k,x,y) fill([x,x+aw,x+aw,x],[y,y,y+ah,y+ah],rgb(k,:),'tag',funname(k),'ButtonDownFcn', @(h) fprintf('%s=''%s'';color=[%.6f,%.6f,%.6f];idx=%d;\n',strname,get (h, 'tag'),get(h,'facecolor'),k));
  end

  hold on
  Height=m*ah+(m-1)*sh;
  x=sw;
  Width=0;
  for j=1:n
    y=Height-ah;
    for i=1:m
      if k<=N
        ColorRec(k,x,y)
        Na=funname(k);
        if length(Na)>tr
          Nan=[Na(1:tr),'*'];
        else
          Nan=Na;
        end
        h=text(x+aw*1.5,y+ah/2,Nan,'HorizontalAlignment','left','Margin',1,'FontSize',fontsize);
        extent=get(h,'extent');
      end
      k=k+1;
      y=y-(ah+sh);
    end
    x=x+aw*1.5+tw+2*sw;
  end
  axis equal
  axis([0 x 0 Height])
  axis off
  title('Mouse clic on color to print rgb ...') 
  set(fig,'visible','on')
  drawnow()
end

function ColorSelected(ObjectH, EventData, H)
  fprintf('%s=''%s'';color=[%.6f,%.6f,%.6f];idx=%d;\n',H{1},get (ObjectH, 'tag'),get(ObjectH,'facecolor'),H{2})
end
